/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

%{C++
#define NSILOCALE_COLLATE "NSILOCALE_COLLATE"
#define NSILOCALE_CTYPE  "NSILOCALE_CTYPE"
#define NSILOCALE_MONETARY "NSILOCALE_MONETARY"
#define NSILOCALE_NUMERIC "NSILOCALE_NUMERIC"
#define NSILOCALE_TIME "NSILOCALE_TIME"
#define NSILOCALE_MESSAGE "NSILOCALE_MESSAGES"

#define NS_LOCALE_CONTRACTID "@mozilla.org/intl/nslocale;1"
%}

/**
 * Represents one locale, which can be used for things like sorting text strings
 * and formatting numbers, dates and times.
 */
[scriptable, uuid(21035ee0-4556-11d3-91cd-00105aa3f7dc)]
interface nsILocale : nsISupports
{
    /**
     * Get the locale code for a given category.
     *
     * A locale code is of the form language[-COUNTRY[-region]], where
     * "language" is an ISO 639 language code (two letter codes preferred over
     * three letter codes when available), "COUNTRY" is an ISO 3166 two letter
     * country code, and "region" is a string of up to 5 letters.
     *
     * A category is one of the following:
     * NSILOCALE_CTYPE: Character classification and case conversion.
     * NSILOCALE_COLLATE: Collation order. How strings are sorted.
     * NSILOCALE_MONETARY: Monetary formatting.
     * NSILOCALE_NUMERIC: Numeric, non-monetary formatting.
     * NSILOCALE_TIME: Date and time formats.
     * NSILOCALE_MESSAGES: Related to fonts, character encodings etc.
     * 
     * @param category
     *        The category of interest.
     * @return The locale code to be used for the given category.
     */
    AString getCategory(in AString category);
};
